@@ -1,19 +1,21 @@
 """Python API for Sensibo."""
-from __future__ import annotations
-import asyncio
-from datetime import datetime, timezone
 
+from __future__ import annotations
+
+import asyncio
 import json
 import logging
+from datetime import datetime, timezone
 from typing import Any
 
-from aiohttp import ClientResponse, ClientSession
+from aiohttp import ClientResponse, ClientSession, ClientTimeout
 
 from .exceptions import AuthenticationError, SensiboError
-from .model import MotionSensor, Schedules, SensiboData, SensiboDevice
+from .model import MotionSensor, PureAQI, Schedules, SensiboData, SensiboDevice
 
 APIV1 = "https://home.sensibo.com/api/v1"
 APIV2 = "https://home.sensibo.com/api/v2"
+API_GRAPHQL = "https://home.sensibo.com/graphql/v1"
 
 TIMEOUT = 5 * 60
 HTTP_AUTH_FAILED_STATUS_CODES = {401, 403}
@@ -35,7 +37,7 @@ class SensiboClient:
         """
         self.api_key = api_key
         self._session = session if session else ClientSession()
-        self.timeout = timeout
+        self.timeout = ClientTimeout(total=timeout)
 
     async def async_get_me(self) -> dict[str, Any]:
         """Return info about me."""
@@ -50,23 +52,31 @@ class SensiboClient:
         params = {"apiKey": self.api_key, "fields": fields}
         return await self._get(APIV2 + "/users/me/pods", params)
 
-    async def async_get_devices_data(self) -> SensiboData:
+    async def async_get_devices_data(self) -> SensiboData:  # noqa: C901
         """Return dataclass with Sensibo Devices."""
-        devices = []
+        devices: list[dict[str, Any]] = []
         data = await self.async_get_devices()
-        for device in data["result"]:
-            devices.append(device)
+        if "result" not in data:
+            LOGGER.warning("No result in data from devices")
+            LOGGER.debug("Data without result: %s", data)
+            raise SensiboError("No result in data")
+        devices = list(data["result"])
 
         device_data: dict[str, SensiboDevice] = {}
         dev: dict[str, Any]
         for dev in devices:
             unique_id = dev["id"]
             mac = dev["macAddress"]
+            serial = dev["serial"]
             name = dev["room"]["name"]
             measure: dict[str, Any] = dev["measurements"]
             temperature = measure.get("temperature")
             feelslike = measure.get("feelsLike")
             humidity = measure.get("humidity")
+            location = dev["location"]["id"]
+            location_name = dev["location"]["name"]
+            auto_off = dev["autoOffEnabled"]
+            auto_off_minutes = dev["autoOffMinutes"]
 
             # Add in new sensors for AirQ + Element model
             tvoc = measure.get("tvoc")
@@ -77,10 +87,32 @@ class SensiboClient:
             iaq = measure.get("iaq")
             rcda = measure.get("rcda")
 
+            # Add AntiMold (premium feature)
+            anti_mold: dict[str, Any] | None = dev["antiMoldConfig"]
+            anti_mold_running = (
+                anti_mold.get("anti_mold_running") if anti_mold else None
+            )
+            anti_mold_enabled = (
+                anti_mold.get("anti_mold_running") if anti_mold else None
+            )
+            anti_mold_fan_time = anti_mold.get("fan_time") if anti_mold else None
+
             ac_states: dict[str, Any] = dev["acState"]
             target_temperature = ac_states.get("targetTemperature")
             hvac_mode = ac_states.get("mode")
             running = ac_states.get("on")
+            available = dev["connectionStatus"].get("isAlive", True)
+
+            capabilities: dict[str, Any] = dev.get("remoteCapabilities", {}) or {}
+            hvac_modes = list(capabilities.get("modes", {}) or {})
+            if not hvac_modes:
+                LOGGER.warning(
+                    "Device %s not correctly registered with remote on Sensibo cloud.",
+                    name,
+                )
+            hvac_modes.append("off")
+            state = hvac_mode if hvac_mode else "off"
+
             fan_mode: str | None = ac_states.get("fanLevel")
             if fan_mode:
                 fan_mode = fan_mode.lower()
@@ -93,39 +125,30 @@ class SensiboClient:
             light_mode: str | None = ac_states.get("light")
             if light_mode:
                 light_mode = light_mode.lower()
-            available = dev["connectionStatus"].get("isAlive", True)
-            capabilities: dict[str, Any] = dev["remoteCapabilities"]
-            hvac_modes = list(capabilities.get("modes", []))
-            if not hvac_modes:
-                LOGGER.warning(
-                    "Device %s not correctly registered with Sensibo cloud. Skipping device",
-                    name,
-                )
-                continue
-            hvac_modes.append("off")
-            current_capabilities: dict[str, Any] = capabilities["modes"][
-                ac_states.get("mode")
-            ]
+
+            current_capabilities: dict[str, Any] = capabilities.get("modes", {}).get(
+                ac_states.get("mode"), {}
+            )
             fan_modes: list[str] | None = current_capabilities.get("fanLevels")
-            fan_modes_to_native: dict | None = None
+            fan_modes_translated: dict[str, str] | None = None
             if fan_modes:
-                fan_modes_to_native = {
+                fan_modes_translated = {
                     _fan_mode.lower(): _fan_mode for _fan_mode in fan_modes
                 }
                 fan_modes = [_fan_mode.lower() for _fan_mode in fan_modes]
             swing_modes: list[str] | None = current_capabilities.get("swing")
-            swing_modes_to_native: dict | None = None
+            swing_modes_translated: dict[str, str] | None = None
             if swing_modes:
-                swing_modes_to_native = {
+                swing_modes_translated = {
                     _swing_mode.lower(): _swing_mode for _swing_mode in swing_modes
                 }
                 swing_modes = [_swing_mode.lower() for _swing_mode in swing_modes]
             horizontal_swing_modes: list[str] | None = current_capabilities.get(
                 "horizontalSwing"
             )
-            horizontal_swing_modes_to_native: dict | None = None
+            horizontal_swing_modes_translated: dict[str, str] | None = None
             if horizontal_swing_modes:
-                horizontal_swing_modes_to_native = {
+                horizontal_swing_modes_translated = {
                     _horizontal_mode.lower(): _horizontal_mode
                     for _horizontal_mode in horizontal_swing_modes
                 }
@@ -134,9 +157,9 @@ class SensiboClient:
                     for _horizontal_mode in horizontal_swing_modes
                 ]
             light_modes: list[str] | None = current_capabilities.get("light")
-            light_modes_to_native: dict | None = None
+            light_modes_translated: dict[str, str] | None = None
             if light_modes:
-                light_modes_to_native = {
+                light_modes_translated = {
                     _light_mode.lower(): _light_mode for _light_mode in light_modes
                 }
                 light_modes = [_light_mode.lower() for _light_mode in light_modes]
@@ -144,54 +167,55 @@ class SensiboClient:
                 "temperatureUnit"
             )
             temperatures_list = (
-                current_capabilities["temperatures"]
+                current_capabilities.get("temperatures", {})
                 .get(temperature_unit_key, {})
                 .get("values", [0, 1])
             )
             if temperatures_list:
                 diff = MAX_POSSIBLE_STEP
                 for i in range(len(temperatures_list) - 1):
-                    if temperatures_list[i + 1] - temperatures_list[i] < diff:
-                        diff = temperatures_list[i + 1] - temperatures_list[i]
+                    diff = min(diff, temperatures_list[i + 1] - temperatures_list[i])
                 temperature_step = diff
 
             active_features = list(ac_states)
             full_features = set()
-            for mode in capabilities["modes"]:
-                if "temperatures" in capabilities["modes"][mode]:
-                    full_features.add("targetTemperature")
-                if "swing" in capabilities["modes"][mode]:
-                    full_features.add("swing")
-                if "fanLevels" in capabilities["modes"][mode]:
-                    full_features.add("fanLevel")
-                if "horizontalSwing" in capabilities["modes"][mode]:
-                    full_features.add("horizontalSwing")
-                if "light" in capabilities["modes"][mode]:
-                    full_features.add("light")
-
-            state = hvac_mode if hvac_mode else "off"
+            if (_capabilities := capabilities.get("modes")) is not None:
+                for mode in _capabilities:
+                    if "temperatures" in _capabilities[mode]:
+                        full_features.add("targetTemperature")
+                    if "swing" in _capabilities[mode]:
+                        full_features.add("swing")
+                    if "fanLevels" in _capabilities[mode]:
+                        full_features.add("fanLevel")
+                    if "horizontalSwing" in _capabilities[mode]:
+                        full_features.add("horizontalSwing")
+                    if "light" in _capabilities[mode]:
+                        full_features.add("light")
 
             fw_ver = dev["firmwareVersion"]
             fw_ver_available = dev.get("currentlyAvailableFirmwareVersion")
             fw_type = dev["firmwareType"]
             model = dev["productModel"]
 
+            # Calibration for temperature and humidity, always available keys
             calibration: dict[str, float] = dev["sensorsCalibration"]
             calibration_temp = calibration.get("temperature")
             calibration_hum = calibration.get("humidity")
 
             # Sky plus supports functionality to use motion sensor as sensor for temp and humidity
+            # In that case the user has it as main sensor we will use those temperature and humidity values
             if main_sensor := dev["mainMeasurementsSensor"]:
                 measurements = main_sensor["measurements"]
                 temperature = measurements.get("temperature")
                 humidity = measurements.get("humidity")
 
             motion_sensors: dict[str, MotionSensor] = {}
-            if dev["motionSensors"]:
+            if dev["motionSensors"]:  # Key is always present
                 sensor: dict[str, Any]
                 for sensor in dev["motionSensors"]:
                     measurement: dict[str, Any] = sensor["measurements"]
                     connection: dict[str, Any] = sensor["connectionStatus"]
+                    # All keys should be present
                     motion_sensors[sensor["id"]] = MotionSensor(
                         id=sensor["id"],
                         alive=connection.get("isAlive"),
@@ -216,6 +240,12 @@ class SensiboClient:
             pure_geo_integration: bool | None = None
             pure_measure_integration: bool | None = None
             pure_prime_integration: bool | None = None
+
+            # Pure devices has special handling for PM2.5 value
+            # as it represents an AQI value
+            pm25 = None
+            pm25_pure = None
+
             if dev["productModel"] == "pure":
                 pure_boost_enabled = pure_conf.get("enabled", False)
                 pure_sensitivity = pure_conf.get("sensitivity", "n").lower()
@@ -225,7 +255,10 @@ class SensiboClient:
                     "measurements_integration", False
                 )
                 pure_prime_integration = pure_conf.get("prime_integration", False)
-            pm25 = measure.get("pm25")
+                if (_pm25 := measure.get("pm25")) is not None:
+                    pm25_pure = PureAQI(_pm25)
+            else:
+                pm25 = measure.get("pm25")
 
             # Binary sensors for main device
             room_occupied = dev["roomIsOccupied"]
@@ -254,7 +287,7 @@ class SensiboClient:
             timer_id = None
             timer_state_on = None
             timer_time = None
-            if dev["productModel"] != "pure":
+            if dev["productModel"] != "pure":  # No timer for Pure devices
                 timer_on = timer.get("isEnabled", False)
                 timer_id = timer.get("id")
                 timer_state: dict[str, Any] | None = timer.get("acState")
@@ -268,18 +301,30 @@ class SensiboClient:
                     else None
                 )
 
-            # Smartmode
+            # Smartmode (climate react)
             smart: dict[str, Any] = dev["smartMode"] if dev["smartMode"] else {}
             smart_on = None
-            if dev["productModel"] != "pure":
+            if dev["productModel"] != "pure":  # No smartmode for Pure devices
                 smart_on = smart.get("enabled", False)
             smart_type: str | None = smart.get("type")
             if smart_type:
                 smart_type = smart_type.lower()
             smart_low_temp_threshold = smart.get("lowTemperatureThreshold")
             smart_high_temp_threshold = smart.get("highTemperatureThreshold")
-            smart_low_state = smart.get("lowTemperatureState")
-            smart_high_state = smart.get("highTemperatureState")
+            _smart_low_state: dict[str, Any] = smart.get("lowTemperatureState", {})
+            _smart_high_state: dict[str, Any] = smart.get("highTemperatureState", {})
+            smart_low_state: dict[str, Any] = {}
+            smart_high_state: dict[str, Any] = {}
+            if _smart_low_state:
+                for key, value in _smart_low_state.items():
+                    smart_low_state[key.lower()] = (
+                        value.lower() if isinstance(value, str) else value
+                    )
+            if _smart_high_state:
+                for key, value in _smart_high_state.items():
+                    smart_high_state[key.lower()] = (
+                        value.lower() if isinstance(value, str) else value
+                    )
 
             # Schedules
             schedule_list = dev["schedules"]
@@ -290,13 +335,20 @@ class SensiboClient:
                     next_utc = datetime.strptime(
                         next_utc_str, "%Y-%m-%dT%H:%M:%S"
                     ).replace(tzinfo=timezone.utc)
+                    _state_full: dict[str, Any] = schedule["acState"]
+                    new_state_full = {}
+                    for key, value in _state_full.items():
+                        new_state_full[key.lower()] = (
+                            value.lower() if isinstance(value, str) else value
+                        )
+                    new_days = [val.lower() for val in schedule["recurringDays"]]
                     schedules[schedule["id"]] = Schedules(
                         id=schedule["id"],
                         enabled=schedule["isEnabled"],
                         name=schedule["name"],
                         state_on=schedule["acState"].get("on"),
-                        state_full=schedule["acState"],
-                        days=schedule["recurringDays"],
+                        state_full=new_state_full,
+                        days=new_days,
                         time=schedule["targetTimeLocal"],
                         next_utc=next_utc,
                     )
@@ -304,6 +356,7 @@ class SensiboClient:
             device_data[unique_id] = SensiboDevice(
                 id=unique_id,
                 mac=mac,
+                serial=serial,
                 name=name,
                 ac_states=ac_states,
                 temp=temperature,
@@ -321,13 +374,13 @@ class SensiboClient:
                 available=available,
                 hvac_modes=hvac_modes,
                 fan_modes=fan_modes,
-                fan_modes_to_native=fan_modes_to_native,
+                fan_modes_translated=fan_modes_translated,
                 swing_modes=swing_modes,
-                swing_modes_to_native=swing_modes_to_native,
+                swing_modes_translated=swing_modes_translated,
                 horizontal_swing_modes=horizontal_swing_modes,
-                horizontal_swing_modes_to_native=horizontal_swing_modes_to_native,
+                horizontal_swing_modes_translated=horizontal_swing_modes_translated,
                 light_modes=light_modes,
-                light_modes_to_native=light_modes_to_native,
+                light_modes_translated=light_modes_translated,
                 temp_unit=temperature_unit_key,
                 temp_list=temperatures_list,
                 temp_step=temperature_step,
@@ -350,6 +403,7 @@ class SensiboClient:
                 pure_prime_integration=pure_prime_integration,
                 pure_conf=pure_conf,
                 pm25=pm25,
+                pm25_pure=pm25_pure,
                 room_occupied=room_occupied,
                 update_available=update_available,
                 filter_clean=filter_clean,
@@ -368,6 +422,13 @@ class SensiboClient:
                 etoh=etoh,
                 iaq=iaq,
                 rcda=rcda,
+                location_id=location,
+                location_name=location_name,
+                anti_mold_running=anti_mold_running,
+                anti_mold_enabled=anti_mold_enabled,
+                anti_mold_fan_time=anti_mold_fan_time,
+                auto_off=auto_off,
+                auto_off_minutes=auto_off_minutes,
             )
 
         return SensiboData(raw=data, parsed=device_data)
@@ -379,7 +440,7 @@ class SensiboClient:
         fields: * for all fields or specific fields
         """
         params = {"apiKey": self.api_key, "fields": fields}
-        return await self._get(APIV2 + "/pods/{}".format(uid), params)
+        return await self._get(APIV2 + f"/pods/{uid}", params)
 
     async def async_reset_filter(self, uid: str) -> dict[str, Any]:
         """Reset filters.
@@ -388,7 +449,7 @@ class SensiboClient:
         """
         params = {"apiKey": self.api_key}
         return await self._delete(
-            APIV2 + "/pods/{}/cleanFiltersNotification".format(uid), params
+            APIV2 + f"/pods/{uid}/cleanFiltersNotification", params
         )
 
     async def async_get_climate_react(self, uid: str) -> dict[str, Any]:
@@ -397,7 +458,7 @@ class SensiboClient:
         uid: UID for device
         """
         params = {"apiKey": self.api_key}
-        return await self._get(APIV2 + "/pods/{}/smartmode".format(uid), params)
+        return await self._get(APIV2 + f"/pods/{uid}/smartmode", params)
 
     async def async_enable_climate_react(
         self, uid: str, data: dict[str, bool]
@@ -408,7 +469,7 @@ class SensiboClient:
         data: dict {enabled: boolean}
         """
         params = {"apiKey": self.api_key}
-        return await self._put(APIV2 + "/pods/{}/smartmode".format(uid), params, data)
+        return await self._put(APIV2 + f"/pods/{uid}/smartmode", params, data)
 
     async def async_set_climate_react(
         self, uid: str, data: dict[str, Any]
@@ -419,7 +480,7 @@ class SensiboClient:
         data: dict according to dev["smartmode"]
         """
         params = {"apiKey": self.api_key}
-        return await self._post(APIV2 + "/pods/{}/smartmode".format(uid), params, data)
+        return await self._post(APIV2 + f"/pods/{uid}/smartmode", params, data)
 
     async def async_get_timer(self, uid: str) -> dict[str, Any]:
         """Get Timer on a device.
@@ -427,7 +488,7 @@ class SensiboClient:
         uid: UID for device
         """
         params = {"apiKey": self.api_key}
-        return await self._get(APIV1 + "/pods/{}/timer/".format(uid), params)
+        return await self._get(APIV1 + f"/pods/{uid}/timer/", params)
 
     async def async_set_timer(self, uid: str, data: dict[str, Any]) -> dict[str, Any]:
         """Set Timer on a device.
@@ -436,7 +497,7 @@ class SensiboClient:
         data: dict according to https://sensibo.github.io/#put-/pods/-device_id-/timer/
         """
         params = {"apiKey": self.api_key}
-        return await self._put(APIV1 + "/pods/{}/timer/".format(uid), params, data)
+        return await self._put(APIV1 + f"/pods/{uid}/timer/", params, data)
 
     async def async_del_timer(self, uid: str) -> dict[str, Any]:
         """Delete Timer on a device.
@@ -444,7 +505,7 @@ class SensiboClient:
         uid: UID for device
         """
         params = {"apiKey": self.api_key}
-        return await self._delete(APIV1 + "/pods/{}/timer/".format(uid), params)
+        return await self._delete(APIV1 + f"/pods/{uid}/timer/", params)
 
     async def async_get_schedules(self, uid: str) -> dict[str, Any]:
         """Get Schedules on a device.
@@ -452,7 +513,7 @@ class SensiboClient:
         uid: UID for device
         """
         params = {"apiKey": self.api_key}
-        return await self._get(APIV1 + "/pods/{}/schedules/".format(uid), params)
+        return await self._get(APIV1 + f"/pods/{uid}/schedules/", params)
 
     async def async_get_schedule(self, uid: str, schedule_id: str) -> dict[str, Any]:
         """Get Schedule on a device.
@@ -461,9 +522,7 @@ class SensiboClient:
         schedule_id: string value for schedule id
         """
         params = {"apiKey": self.api_key}
-        return await self._get(
-            APIV1 + "/pods/{}/schedules/{}".format(uid, schedule_id), params
-        )
+        return await self._get(APIV1 + f"/pods/{uid}/schedules/{schedule_id}", params)
 
     async def async_set_schedule(
         self, uid: str, data: dict[str, Any]
@@ -475,7 +534,7 @@ class SensiboClient:
         data: dict according to https://sensibo.github.io/#post-/pods/-device_id-/schedules/
         """
         params = {"apiKey": self.api_key}
-        return await self._post(APIV1 + "/pods/{}/schedules/".format(uid), params, data)
+        return await self._post(APIV1 + f"/pods/{uid}/schedules/", params, data)
 
     async def async_enable_schedule(
         self, uid: str, schedule_id: str, data: dict[str, bool]
@@ -488,7 +547,7 @@ class SensiboClient:
         """
         params = {"apiKey": self.api_key}
         return await self._put(
-            APIV1 + "/pods/{}/schedules/{}".format(uid, schedule_id), params, data
+            APIV1 + f"/pods/{uid}/schedules/{schedule_id}", params, data
         )
 
     async def async_del_schedule(self, uid: str, schedule_id: str) -> dict[str, Any]:
@@ -499,7 +558,7 @@ class SensiboClient:
         """
         params = {"apiKey": self.api_key}
         return await self._delete(
-            APIV1 + "/pods/{}/schedules/{}".format(uid, schedule_id), params
+            APIV1 + f"/pods/{uid}/schedules/{schedule_id}", params
         )
 
     async def async_set_calibration(
@@ -511,9 +570,7 @@ class SensiboClient:
         data: dict temperature or humidity and float as value
         """
         params = {"apiKey": self.api_key}
-        return await self._post(
-            APIV2 + "/pods/{}/calibration/".format(uid), params, data
-        )
+        return await self._post(APIV2 + f"/pods/{uid}/calibration/", params, data)
 
     async def async_set_pureboost(
         self, uid: str, data: dict[str, float]
@@ -524,7 +581,7 @@ class SensiboClient:
         data: dict as pure_conf
         """
         params = {"apiKey": self.api_key}
-        return await self._put(APIV2 + "/pods/{}/pureboost".format(uid), params, data)
+        return await self._put(APIV2 + f"/pods/{uid}/pureboost", params, data)
 
     async def async_set_ac_states(
         self,
@@ -538,7 +595,7 @@ class SensiboClient:
         """
         params = {"apiKey": self.api_key}
         data = {"acState": ac_state}
-        return await self._post(APIV2 + "/pods/{}/acStates".format(uid), params, data)
+        return await self._post(APIV2 + f"/pods/{uid}/acStates", params, data)
 
     async def async_set_ac_state_property(
         self,
@@ -560,9 +617,7 @@ class SensiboClient:
         data = {"currentAcState": ac_state, "newValue": value}
         if assumed_state:
             data["reason"] = "StateCorrectionByUser"
-        return await self._patch(
-            APIV2 + "/pods/{}/acStates/{}".format(uid, name), params, data
-        )
+        return await self._patch(APIV2 + f"/pods/{uid}/acStates/{name}", params, data)
 
     async def _get(
         self, path: str, params: dict[str, Any], retry: int = 3
@@ -574,12 +629,12 @@ class SensiboClient:
                 path, params=params, timeout=self.timeout
             ) as resp:
                 return await self._response(resp)
-        except Exception as error:
+        except Exception:
             LOGGER.debug("Retry %d on path %s", 4 - retry, path)
             if retry > 0:
                 await asyncio.sleep(7)
                 return await self._get(path, params, retry - 1)
-            raise error
+            raise
 
     async def _put(
         self,
@@ -594,11 +649,11 @@ class SensiboClient:
                 path, params=params, data=json.dumps(data), timeout=self.timeout
             ) as resp:
                 return await self._response(resp)
-        except Exception as error:
+        except Exception:
             if retry is False:
                 await asyncio.sleep(5)
                 return await self._put(path, params, data, True)
-            raise error
+            raise
 
     async def _post(
         self,
@@ -614,11 +669,11 @@ class SensiboClient:
                 path, params=params, data=json.dumps(data), timeout=self.timeout
             ) as resp:
                 return await self._response(resp)
-        except Exception as error:
+        except Exception:
             if retry is False:
                 await asyncio.sleep(5)
                 return await self._post(path, params, data, True)
-            raise error
+            raise
 
     async def _patch(
         self,
@@ -634,11 +689,11 @@ class SensiboClient:
                 path, params=params, data=json.dumps(data), timeout=self.timeout
             ) as resp:
                 return await self._response(resp)
-        except Exception as error:
+        except Exception:
             if retry is False:
                 await asyncio.sleep(5)
                 return await self._patch(path, params, data, True)
-            raise error
+            raise
 
     async def _delete(
         self, path: str, params: dict[str, Any], retry: bool = False
@@ -650,11 +705,11 @@ class SensiboClient:
                 path, params=params, timeout=self.timeout
             ) as resp:
                 return await self._response(resp)
-        except Exception as error:
+        except Exception:
             if retry is False:
                 await asyncio.sleep(5)
                 return await self._delete(path, params, True)
-            raise error
+            raise
 
     async def _response(self, resp: ClientResponse) -> dict[str, Any]:
         """Return response from call."""
